import torch
from logger import Logger
LOGGER = Logger().logger()
from torchnlp.word_to_vector import GloVe, FastText, BPEmb, CharNGram
from Utils import device
#Author: @meetdoshi
class WordVectors():
    def __init__(self,model='fasttext'):
        '''
        Init function
        @params
        model: str {fasttext,glove,bpemb,chargram}
        '''
        self.vectors = None
        if model=='fasttext':
            self.vectors = FastText()
        elif model=='glove':
            self.vectors = GloVe()
        elif model=='bpemb':
            self.vectors = BPEmb()
        elif model=='chargram':
            self.vectors = CharNGram()
        else:
            LOGGER.error("Model not found")
        return

    def get_vectors(self):
        '''
        Function to get all vectors by a model
        '''
        if(self.vectors == None):
            LOGGER.error("Model not found")
        return self.vectors

    def get_vector(self,word):
        '''
        Function to get vector for a word
        @params
        word: str
        '''
        if(self.vectors == None):
            LOGGER.error("Model not found")
        return self.vectors[word]

''' 
if __name__ == "__main__":
    wv = WordVectors()
    print(wv.get_vector('ababasba').shape)
    wv = WordVectors(model='glove')
    print(wv.get_vector('ababababas').shape)
    wv = WordVectors(model='bpemb')
    print(wv.get_vector('hbasbasbsello').shape)
    wv = WordVectors(model='chargram')
    print(wv.get_vector('hebasallo').shape)
'''